#ifndef __S3C_UART_H
#define __S3C_UART_H

/* register offset */
#define ULCON       0x00
#define UCON        0x04
#define UFCON       0x08
#define UMCON       0x0c
#define UTRSTAT     0x10
#define UERSTAT     0x14
#define UFSTAT      0x18
#define UMSTAT      0x1c
#define UTXH        0x20
#define URXH        0x24
#define UBRDIV      0x28
#define UDIVSLOT    0x2c

/* ULCON */
#define ULCON_IR_MODE       (1 << 6)
#define ULCON_PARITY_NO     (0)
#define ULCON_PARITY_ODD    (4 << 3)
#define ULCON_PARITY_EVEN   (5 << 3)
#define ULCON_PARITY_FORCE1 (6 << 3)
#define ULCON_PARITY_FORCE0 (7 << 3)
#define ULCON_STOPBIT_1     (0)
#define ULCON_STOPBIT_2     (1 << 2)
#define ULCON_WL_5BITS      (0)
#define ULCON_WL_6BITS      (1)
#define ULCON_WL_7BITS      (2)
#define ULCON_WL_8BITS      (3)
#define ULCON_8N1           (ULCON_PARITY_NO | ULCON_STOPBIT_1 | ULCON_WL_8BITS)

/* UCON */
#define UCON_CLK_PCLK       (0)
#define UCON_CLK_EXTUARTCLK (1 << 10)
#define UCON_CLK_DIVEPLL    (3 << 10)
#define UCON_TXIRQ_PULSE    (0)
#define UCON_TXIRQ_LEVEL    (1 << 9)
#define UCON_RXIRQ_PULSE    (0)
#define UCON_RXIRQ_LEVEL    (1 << 8)
#define UCON_RXTMOUT_ENABLE (1 << 7)
#define UCON_ERRIRQ_ELABLE  (1 << 6)
#define UCON_LOOPBACK       (1 << 5)
#define UCON_SEND_BREAK     (1 << 4)
#define UCON_TXMODE_IRQ     (1 << 2)
#define UCON_TXMODE_DMA0    (2 << 2)
#define UCON_TXMODE_DMA1    (3 << 2)
#define UCON_RXMODE_IRQ     (1)
#define UCON_RXMODE_DMA0    (2)
#define UCON_RXMODE_DMA1    (3)

/* UFCON */
#define UFCON_TXTRG_EMPTY   (0)
#define UFCON_TXTRG_16BYTES (1 << 6)
#define UFCON_TXTRG_32BYTES (2 << 6)
#define UFCON_TXTRG_48BYTES (3 << 6)
#define UFCON_RXTRG_1BYTES  (0)
#define UFCON_RXTRG_8BYTES  (1 << 4)
#define UFCON_RXTRG_16BYTES (2 << 4)
#define UFCON_RXTRG_32BYTES (3 << 4)
#define UFCON_TXFIFO_RESET  (1 << 2)
#define UFCON_RXFIFO_RESET  (1 << 1)
#define UFCON_FIFO_ENABLE   (1)

/* UFSTAT */
#define UFSTAT_TXFIFO_FULL       (1 << 14)
#define UFSTAT_TXFIFO_COUNT(x)   ((x >> 8) & 0x3f)
#define UFSTAT_RXFIFO_FULL       (1 << 6)
#define UFSTAT_RXFIFO_COUNT(x)   (x & 0x3f)

void mango_uart_init(unsigned long base, int baud);

int mango_uart_write(const char *buf, int len);

#endif  /* __S3C_UART_H */
