#include <stdio.h>
#include <string.h>
#include <s3c_uart.h>
#include <s3c_timer.h>
#include <mango_button.h>
#include <mango_backlight.h>

static void mango_hw_init(void)
{
    mango_timer_init();
    mango_uart_init(1, 115200);
    mango_btn_init();
    mango_backlight_init();
}

static void mango_print_splash(void)
{
    printf("\n");
    printf("Mango Firmware - %s %s\n\n", __TIME__, __DATE__);
}

static void mango_menu_button(void)
{
    int len;
    char buf;

    printf("To exit button test, press any key (in keyboard)\n");

    while (1) {
        len = mango_uart_read(&buf, 1, 1);
        if (len > 0)
            break;

        if (mango_btn_scan(MANGO_BTN_KEYCODE_UP) == MANGO_BTN_STATE_DOWN)
            printf("Up key pressed\n");

        if (mango_btn_scan(MANGO_BTN_KEYCODE_DOWN) == MANGO_BTN_STATE_DOWN)
            printf("Down key pressed\n");

        if (mango_btn_scan(MANGO_BTN_KEYCODE_LEFT) == MANGO_BTN_STATE_DOWN)
            printf("Left key pressed\n");

        if (mango_btn_scan(MANGO_BTN_KEYCODE_RIGHT) == MANGO_BTN_STATE_DOWN)
            printf("Right key pressed\n");

        mdelay(50);
    }
}

static void mango_menu_timer(void)
{
    int i;
    for(i = 0; i < 10; i ++) {
        mdelay(1000);
        printf("1 Second Delay Test %d\n", i);
    }
}

static void mango_menu_backlight(void)
{
    int c;

    while (1) {
        printf("\nLCD Backlight Control\n");
        printf("1> Step Up LCD Brightness\n");
        printf("2> Step Down LCD Brightness\n");
        printf("x> Exit to back\n");
        c = getchar();
        printf("%c is selected\n", c);

        switch (c) {
        case '1':
            mango_backlight_stepup();
            break;
        case '2':
            mango_backlight_stepdown();
            break;
        case 'x':
            goto finished;
        }
    }
finished:
	return;
}

static void mango_menu_main(void)
{
    int c;

    while (1) {
        printf("\nMain menu\n");
        printf("1> Button Test\n");
        printf("2> Timer Test\n");
        printf("3> LCD Backlight Test\n");
        printf("x> Exit\n");
        c = getchar();
        printf("%c is selected\n", c);

        switch (c) {
        case '1':
            mango_menu_button();
            break;
        case '2':
            mango_menu_timer();
            break;
        case '3':
            mango_menu_backlight();
            break;
        case 'x':
            goto finished;
        default:
            printf("No item\n");
            break;
        }
    }
finished:
	return;
}

int main()
{
    mango_hw_init();

    mango_print_splash();

    mango_menu_main();

    return 0;
}

